@@ -21,46 +21,42 @@ import docking.widgets.fieldpanel.field.Field;
 import docking.widgets.fieldpanel.support.FieldLocation;
 import ghidra.GhidraOptions;
 import ghidra.app.plugin.core.hover.AbstractReferenceHover;
-import ghidra.app.util.HelpTopics;
-import ghidra.framework.options.Options;
 import ghidra.framework.plugintool.PluginTool;
 import ghidra.program.model.address.Address;
 import ghidra.program.model.listing.Function;
 import ghidra.program.model.listing.Program;
 import ghidra.program.util.ProgramLocation;
-import ghidra.util.HelpLocation;
 
 public class ReferenceDecompilerHover extends AbstractReferenceHover
 		implements DecompilerHoverService {
 
-	protected static final String NAME = "Decompiler Reference Viewer";
-
+	private static final String NAME = "Decompiler Reference Viewer";
 	private static final String DESCRIPTION =
 		"Shows \"referred to\" code and data from the decompiler.";
 
-	private static final int PRIORITY = 25;
+	// note: this is relative to other DecompilerHovers; a higher priority gets called first
+	// Use high value so this hover gets called first.  The method for determining what the user
+	// is hovering in the Decompiler is less then perfect.  We choose to allow the more precise
+	// hovers get a chance to process the request first.
+	private static final int PRIORITY = 50;
 
 	public ReferenceDecompilerHover(PluginTool tool) {
 		super(tool, PRIORITY);
 	}
 
 	@Override
-	public void initializeOptions() {
-		options = tool.getOptions(GhidraOptions.CATEGORY_DECOMPILER_POPUPS);
+	protected String getName() {
+		return NAME;
+	}
 
-		options.setOptionsHelpLocation(new HelpLocation(HelpTopics.CODE_BROWSER, "MouseHover"));
-		HelpLocation help = new HelpLocation(HelpTopics.CODE_BROWSER, "ReferenceHover");
-		options.getOptions(NAME).setOptionsHelpLocation(help);
+	@Override
+	protected String getDescription() {
+		return DESCRIPTION;
+	}
 
-		options.registerOption(NAME, true, null, DESCRIPTION);
-
-		options.registerOption(NAME + Options.DELIMITER + "Dialog Height", 400, help,
-			"Height of the popup window");
-		options.registerOption(NAME + Options.DELIMITER + "Dialog Width", 600, help,
-			"Width of the popup window");
-
-		setOptions(options, NAME);
-		options.addOptionsChangeListener(this);
+	@Override
+	protected String getOptionsCategory() {
+		return GhidraOptions.CATEGORY_DECOMPILER_POPUPS;
 	}
 
 	@Override
