@@ -4,9 +4,9 @@
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -18,23 +18,29 @@ package ghidra.app.plugin.core.progmgr;
 import java.awt.event.InputEvent;
 import java.awt.event.KeyEvent;
 
-import javax.swing.KeyStroke;
-import javax.swing.Timer;
+import javax.swing.*;
 
 import docking.ActionContext;
 import docking.DockingUtils;
 import docking.action.*;
+import docking.action.builder.ActionBuilder;
 import docking.tool.ToolConstants;
+import docking.widgets.tab.GTabPanel;
+import generic.theme.GIcon;
 import ghidra.app.CorePluginPackage;
 import ghidra.app.events.*;
 import ghidra.app.plugin.PluginCategoryNames;
 import ghidra.app.services.CodeViewerService;
 import ghidra.app.services.ProgramManager;
 import ghidra.framework.model.*;
+import ghidra.framework.options.OptionsChangeListener;
+import ghidra.framework.options.ToolOptions;
 import ghidra.framework.plugintool.*;
 import ghidra.framework.plugintool.util.PluginStatus;
 import ghidra.program.model.listing.Program;
 import ghidra.util.HelpLocation;
+import ghidra.util.bean.opteditor.OptionsVetoException;
+import help.Help;
 
 /**
  * Plugin to show a "tab" for each open program; the selected tab is the activated program.
@@ -52,19 +58,23 @@ import ghidra.util.HelpLocation;
 	eventsConsumed = { ProgramOpenedPluginEvent.class, ProgramClosedPluginEvent.class, ProgramActivatedPluginEvent.class, ProgramVisibilityChangePluginEvent.class }
 )
 //@formatter:on
-public class MultiTabPlugin extends Plugin implements DomainObjectListener {
+public class MultiTabPlugin extends Plugin implements DomainObjectListener, OptionsChangeListener {
+	private final static Icon TRANSIENT_ICON = new GIcon("icon.plugin.programmanager.transient");
+	private final static Icon EMPTY8_ICON = new GIcon("icon.plugin.programmanager.empty.small");
+	private static final String SHOW_TABS_ALWAYS = "Show Program Tabs Always";
 
 	//
-	// Unusual Code Alert!: We can't initialize these in the fields above because calling
+	// Unusual Code Alert!: We can't initialize these fields below because calling
 	// DockingUtils calls into Swing code.  Further, we don't want Swing code being accessed
 	// when the Plugin classes are loaded, as they get loaded in the headless environment.
-	// 
+	// So these fields are not static.
+	//
 	private final KeyStroke NEXT_TAB_KEYSTROKE =
 		KeyStroke.getKeyStroke(KeyEvent.VK_F9, DockingUtils.CONTROL_KEY_MODIFIER_MASK);
 	private final KeyStroke PREVIOUS_TAB_KEYSTROKE =
 		KeyStroke.getKeyStroke(KeyEvent.VK_F8, DockingUtils.CONTROL_KEY_MODIFIER_MASK);
 
-	private MultiTabPanel tabPanel;
+	private GTabPanel<Program> tabPanel;
 	private ProgramManager progService;
 	private CodeViewerService cvService;
 	private DockingAction goToProgramAction;
@@ -78,12 +88,32 @@ public class MultiTabPlugin extends Plugin implements DomainObjectListener {
 
 	public MultiTabPlugin(PluginTool tool) {
 		super(tool);
-
 		createActions();
 	}
 
 	private void createActions() {
 
+		new ActionBuilder("Close Program", getName())
+				.popupMenuPath("Close")
+				.helpLocation(new HelpLocation("ProgramManagerPlugin", "Close_Program"))
+				.withContext(ProgramTabActionContext.class)
+				.onAction(c -> progService.closeProgram(c.getProgram(), false))
+				.buildAndInstall(tool);
+
+		new ActionBuilder("Close Other Programs", getName())
+				.popupMenuPath("Close Others")
+				.helpLocation(new HelpLocation("ProgramManagerPlugin", "Close_Others"))
+				.withContext(ProgramTabActionContext.class)
+				.onAction(c -> closeOtherPrograms(c.getProgram()))
+				.buildAndInstall(tool);
+
+		new ActionBuilder("Close All Programs", getName())
+				.popupMenuPath("Close All")
+				.helpLocation(new HelpLocation("ProgramManagerPlugin", "Close_All"))
+				.withContext(ProgramTabActionContext.class)
+				.onAction(c -> progService.closeAllPrograms(false))
+				.buildAndInstall(tool);
+
 		String firstGroup = "1";
 		String secondGroup = "2";
 
@@ -96,20 +126,20 @@ public class MultiTabPlugin extends Plugin implements DomainObjectListener {
 		goToProgramAction.setMenuBarData(
 			new MenuData(new String[] { ToolConstants.MENU_NAVIGATION, "Go To Program..." }, null,
 				ToolConstants.MENU_NAVIGATION_GROUP_WINDOWS, MenuData.NO_MNEMONIC, firstGroup));
-		goToProgramAction.setKeyBindingData(
-			new KeyBindingData(KeyEvent.VK_F7, InputEvent.CTRL_DOWN_MASK));
+		goToProgramAction
+				.setKeyBindingData(new KeyBindingData(KeyEvent.VK_F7, InputEvent.CTRL_DOWN_MASK));
 
 		goToProgramAction.setEnabled(false);
 		goToProgramAction.setDescription(
 			"Shows the program selection dialog with the current program selected");
-		goToProgramAction.setHelpLocation(
-			new HelpLocation("ProgramManagerPlugin", "Go_To_Program"));
+		goToProgramAction
+				.setHelpLocation(new HelpLocation("ProgramManagerPlugin", "Go_To_Program"));
 
 		goToNextProgramAction = new DockingAction("Go To Next Program", getName()) {
 			@Override
 			public void actionPerformed(ActionContext context) {
 				// highlight the next tab
-				nextProgramPressed();
+				cycleNextProgram(true);
 			}
 		};
 		goToNextProgramAction.setEnabled(false);
@@ -123,7 +153,7 @@ public class MultiTabPlugin extends Plugin implements DomainObjectListener {
 			@Override
 			public void actionPerformed(ActionContext context) {
 				// highlight the previous tab
-				previousProgramPressed();
+				cycleNextProgram(false);
 			}
 		};
 		goToPreviousProgramAction.setEnabled(false);
@@ -133,7 +163,7 @@ public class MultiTabPlugin extends Plugin implements DomainObjectListener {
 		goToPreviousProgramAction.setHelpLocation(
 			new HelpLocation("ProgramManagerPlugin", "Go_To_Next_And_Previous_Program"));
 
-		// this timer is to give the user time to select successive programs before activating one 
+		// this timer is to give the user time to select successive programs before activating one
 		selectHighlightedProgramTimer = new Timer(750, e -> selectHighlightedProgram());
 		selectHighlightedProgramTimer.setRepeats(false);
 
@@ -146,11 +176,11 @@ public class MultiTabPlugin extends Plugin implements DomainObjectListener {
 		goToLastActiveProgramAction.setMenuBarData(new MenuData(
 			new String[] { ToolConstants.MENU_NAVIGATION, "Go To Last Active Program" }, null,
 			ToolConstants.MENU_NAVIGATION_GROUP_WINDOWS, MenuData.NO_MNEMONIC, secondGroup));
-		goToLastActiveProgramAction.setKeyBindingData(
-			new KeyBindingData(KeyEvent.VK_F6, InputEvent.CTRL_DOWN_MASK));
+		goToLastActiveProgramAction
+				.setKeyBindingData(new KeyBindingData(KeyEvent.VK_F6, InputEvent.CTRL_DOWN_MASK));
 		goToLastActiveProgramAction.setEnabled(false);
-		goToLastActiveProgramAction.setDescription(
-			"Activates the last program used before the current program");
+		goToLastActiveProgramAction
+				.setDescription("Activates the last program used before the current program");
 		goToLastActiveProgramAction.setHelpLocation(
 			new HelpLocation("ProgramManagerPlugin", "Go_To_Last_Active_Program"));
 
@@ -160,6 +190,11 @@ public class MultiTabPlugin extends Plugin implements DomainObjectListener {
 		tool.addAction(goToPreviousProgramAction);
 	}
 
+	private void closeOtherPrograms(Program keepProgram) {
+		progService.setCurrentProgram(keepProgram);
+		progService.closeOtherPrograms(false);
+	}
+
 	private void updateActionEnablement() {
 		// the next/previous actions should not be enabled if no tabs are hidden
 		boolean enable = (tabPanel.getTabCount() > 1);
@@ -173,89 +208,96 @@ public class MultiTabPlugin extends Plugin implements DomainObjectListener {
 
 	private void switchToProgram(Program program) {
 		if (lastActiveProgram != null) {
-			tabPanel.setSelectedProgram(lastActiveProgram);
+			tabPanel.selectTab(lastActiveProgram);
 		}
 	}
 
 	private void showProgramList() {
-		tabPanel.showProgramList();
-	}
-
-	private void highlightNextProgram(boolean forwardDirection) {
-		tabPanel.highlightNextProgram(forwardDirection);
+		tabPanel.showTabList(!tabPanel.isShowingTabList());
 	}
 
 	private void selectHighlightedProgram() {
-		tabPanel.selectHighlightedProgram();
-	}
-
-	String getStringUsedInList(Program program) {
-		DomainFile df = program.getDomainFile();
-		String changeIndicator = program.isChanged() ? "*" : "";
-		if (!df.isInWritableProject()) {
-			return df.toString() + " [Read-Only]" + changeIndicator;
+		Program highlightedTabValue = tabPanel.getHighlightedTabValue();
+		if (highlightedTabValue != null) {
+			tabPanel.selectTab(highlightedTabValue);
 		}
-		return df.toString() + changeIndicator;
 	}
 
-	String getToolTip(Program program) {
-		return getStringUsedInList(program);
+	private String getToolTip(Program program) {
+		return DomainObjectDisplayUtils.getToolTip(program);
 	}
 
-	String getName(Program program) {
-		DomainFile df = program.getDomainFile();
-		String tabName = df.getName();
-		if (df.isReadOnly()) {
-			int version = df.getVersion();
-			if (!df.canSave() && version != DomainFile.DEFAULT_VERSION) {
-				tabName += "@" + version;
-			}
-			tabName = tabName + " [Read-Only]";
-		}
-		return tabName;
+	private String getTabName(Program program) {
+		return DomainObjectDisplayUtils.getTabText(program);
 	}
 
 	void keyTypedFromListWindow(KeyEvent e) {
 
 		KeyStroke stroke = KeyStroke.getKeyStrokeForEvent(e);
 		if (stroke.equals(NEXT_TAB_KEYSTROKE)) {
-			nextProgramPressed();
+			cycleNextProgram(true);
 		}
 		else if (stroke.equals(PREVIOUS_TAB_KEYSTROKE)) {
-			previousProgramPressed();
+			cycleNextProgram(false);
 		}
 	}
 
-	private void nextProgramPressed() {
-		highlightNextProgram(true);
+	private void cycleNextProgram(boolean forward) {
+		tabPanel.highlightNextPreviousTab(forward);
 		selectHighlightedProgramTimer.restart();
 	}
 
-	private void previousProgramPressed() {
-		highlightNextProgram(false);
-		selectHighlightedProgramTimer.restart();
-	}
-
-	boolean isChanged(Object obj) {
-		return ((Program) obj).isChanged();
-	}
-
 	@Override
 	public void domainObjectChanged(DomainObjectChangedEvent ev) {
 		if (ev.getSource() instanceof Program) {
 			Program program = (Program) ev.getSource();
-			tabPanel.refresh(program);
+			tabPanel.refreshTab(program);
 		}
 	}
 
 	@Override
 	protected void init() {
-		tabPanel = new MultiTabPanel(this);
+		tabPanel = new GTabPanel<Program>("Program");
+		tabPanel.setNameFunction(p -> getTabName(p));
+		tabPanel.setIconFunction(p -> getIcon(p));
+		tabPanel.setToolTipFunction(p -> getToolTip(p));
+		tabPanel.setSelectedTabConsumer(p -> programSelected(p));
+		tabPanel.setCloseTabConsumer(p -> progService.closeProgram(p, false));
+		Help.getHelpService()
+				.registerHelp(tabPanel, new HelpLocation("ProgramManagerPlugin", "Navigate_File"));
+
+		initOptions();
+
 		progService = tool.getService(ProgramManager.class);
 		cvService = tool.getService(CodeViewerService.class);
 		cvService.setNorthComponent(tabPanel);
 	}
 
+	private void initOptions() {
+		ToolOptions options = tool.getOptions(ToolConstants.TOOL_OPTIONS);
+		options.registerOption(SHOW_TABS_ALWAYS, false, null,
+			"If true, program tabs will be displayed even if only one");
+
+		tabPanel.setShowTabsAlways(options.getBoolean(SHOW_TABS_ALWAYS, false));
+		options.addOptionsChangeListener(this);
+	}
+
+	@Override
+	public void optionsChanged(ToolOptions options, String optionName, Object oldValue,
+			Object newValue) throws OptionsVetoException {
+		if (optionName.equals(SHOW_TABS_ALWAYS)) {
+			tabPanel.setShowTabsAlways((Boolean) newValue);
+		}
+	}
+
+	private Icon getIcon(Program program) {
+		ProjectLocator projectLocator = program.getDomainFile().getProjectLocator();
+		if (projectLocator != null && projectLocator.isTransient()) {
+			return TRANSIENT_ICON;
+		}
+		return EMPTY8_ICON;
+	}
+
 	boolean removeProgram(Program program) {
 		return progService.closeProgram(program, false);
 	}
@@ -263,13 +305,14 @@ public class MultiTabPlugin extends Plugin implements DomainObjectListener {
 	void programSelected(Program program) {
 		if (program != progService.getCurrentProgram()) {
 			progService.setCurrentProgram(program);
+			cvService.requestFocus();
 		}
 	}
 
 	private void add(Program prog) {
 
 		if (progService.isVisible(prog)) {
-			tabPanel.addProgram(prog);
+			tabPanel.addTab(prog);
 			prog.removeListener(this);
 			prog.addListener(this);
 			updateActionEnablement();
@@ -278,7 +321,7 @@ public class MultiTabPlugin extends Plugin implements DomainObjectListener {
 
 	private void remove(Program prog) {
 		prog.removeListener(this);
-		tabPanel.removeProgram(prog);
+		tabPanel.removeTab(prog);
 		updateActionEnablement();
 	}
 
@@ -305,8 +348,8 @@ public class MultiTabPlugin extends Plugin implements DomainObjectListener {
 
 			if (prog != null) {
 				add(prog);
-				if (tabPanel.getSelectedProgram() != prog) {
-					tabPanel.setSelectedProgram(prog);
+				if (tabPanel.getSelectedTabValue() != prog) {
+					tabPanel.selectTab(prog);
 					updateActionEnablement();
 				}
 			}
@@ -317,7 +360,7 @@ public class MultiTabPlugin extends Plugin implements DomainObjectListener {
 				add(prog);
 				if (progService.getCurrentProgram() != prog) {
 					currentProgram = prog;
-					tabPanel.setSelectedProgram(prog);
+					tabPanel.selectTab(prog);
 					updateActionEnablement();
 				}
 			}
