@@ -4,9 +4,9 @@
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -57,7 +57,7 @@ class ApplyStackVariables {
 		boolean skipParameters = !isParameterRecoverySupported();
 
 		while (xmlParser.hasNext()) {
-			monitor.checkCanceled();
+			monitor.checkCancelled();
 
 			XmlElement elem = xmlParser.peek();
 			if (elem.isEnd() && elem.getName().equals("function")) {
@@ -134,7 +134,7 @@ class ApplyStackVariables {
 			function.getProgram().getListing().getInstructions(scopeSet, true);
 		int max = 0;
 		while (instructions.hasNext()) {
-			monitor.checkCanceled();
+			monitor.checkCancelled();
 			Instruction next = instructions.next();
 			int newValue = valueChange.getDepth(next.getMinAddress());
 			if (newValue < -(20 * 1024) || newValue > (20 * 1024)) {
@@ -222,7 +222,7 @@ class ApplyStackVariables {
 			return null;
 		}
 		if (wrappedDataType.isZeroLengthArray()) {
-			log.appendMsg("PDB", "Zero length array not supported for for " + member.kind + ": " +
+			log.appendMsg("PDB", "Zero length array not supported for " + member.kind + ": " +
 				member.memberDataTypeName);
 			return null;
 		}
