@@ -4,9 +4,9 @@
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -45,8 +45,8 @@ import generic.random.SecureRandomFactory;
 import ghidra.framework.Application;
 import ghidra.framework.ApplicationConfiguration;
 import ghidra.framework.remote.*;
-import ghidra.net.ApplicationKeyManagerFactory;
-import ghidra.net.SSLContextInitializer;
+import ghidra.net.DefaultKeyManagerFactory;
+import ghidra.net.DefaultSSLContextInitializer;
 import ghidra.server.RepositoryManager;
 import ghidra.server.UserManager;
 import ghidra.server.security.*;
@@ -55,7 +55,6 @@ import ghidra.server.stream.RemoteBlockStreamHandle;
 import ghidra.util.SystemUtilities;
 import ghidra.util.exception.AssertException;
 import ghidra.util.exception.DuplicateNameException;
-import resources.ResourceManager;
 import utilities.util.FileUtilities;
 import utility.application.ApplicationLayout;
 
@@ -67,13 +66,21 @@ import utility.application.ApplicationLayout;
  */
 public class GhidraServer extends UnicastRemoteObject implements GhidraServerHandle {
 
+	private static final String SERIAL_FILTER_FILE = "serial.filter";
+
+	private static final String TLS_SERVER_PROTOCOLS_PROPERTY = "ghidra.tls.server.protocols";
+	private static final String TLS_ENABLED_CIPHERS_PROPERTY = "jdk.tls.server.cipherSuites";
+
+	private static final String SERIALIZATION_FILTER_DISABLED_PROPERTY =
+		"ghidra.server.serialization.filter.disabled";
+
 	private static SslRMIServerSocketFactory serverSocketFactory;
 	private static SslRMIClientSocketFactory clientSocketFactory;
 	private static InetAddress bindAddress;
 
 	private static Logger log;
 
-	private static String HELP_FILE = "/ghidra/server/remote/ServerHelp.txt";
+	private static String HELP_FILE = "ServerHelp.txt";
 	private static String USAGE_ARGS =
 		"[-ip <hostname>] [-i #.#.#.#] [-p#] [-n] [-a#] [-d<ad_domain>] [-e<days>] [-jaas <config_file>] [-u] [-autoProvision] [-anonymous] [-ssh] <repository_path>";
 
@@ -135,7 +142,9 @@ public class GhidraServer extends UnicastRemoteObject implements GhidraServerHan
 	 * @param allowAnonymousAccess allow anonymous access if true
 	 * @param autoProvisionAuthedUsers flag to turn on automatically adding successfully
 	 * authenticated users to the user manager if they don't already exist
-	 * @throws IOException
+	 * @param jaasConfigFile JAAS configuration file
+	 * @throws IOException if an IO error occurs
+	 * @throws CertificateException if failed to parse CA certs file used for PKI authentication
 	 */
 	GhidraServer(File rootDir, AuthMode authMode, String loginDomain,
 			boolean allowUserToSpecifyName, boolean altSSHLoginAllowed,
@@ -158,11 +167,9 @@ public class GhidraServer extends UnicastRemoteObject implements GhidraServerHan
 		}
 
 		boolean supportLocalPasswords = false;
-		boolean requireExplicitPasswordReset = true;
 		switch (authMode) {
 			case PASSWORD_FILE_LOGIN:
 				supportLocalPasswords = true;
-				requireExplicitPasswordReset = false;
 				authModule = new PasswordFileAuthenticationModule(allowUserToSpecifyName);
 				break;
 			case PKI_LOGIN:
@@ -199,11 +206,14 @@ public class GhidraServer extends UnicastRemoteObject implements GhidraServerHan
 			sshAuthModule = new SSHAuthenticationModule(allowUserToSpecifyName);
 		}
 
-		mgr = new RepositoryManager(rootDir, supportLocalPasswords, requireExplicitPasswordReset,
-			defaultPasswordExpirationDays, allowAnonymousAccess);
+		mgr = new RepositoryManager(rootDir, supportLocalPasswords, defaultPasswordExpirationDays,
+			allowAnonymousAccess);
 
 		GhidraServer.server = this;
 
+		// Establish serialization filter to address deserialization vulnerabity concerns.
+		setGlobalSerializationFilter();
+
 		// Start block stream server - use RMI serverSocketFactory
 		blockStreamServer = BlockStreamServer.getBlockStreamServer();
 		ServerSocket streamServerSocket;
@@ -240,12 +250,12 @@ public class GhidraServer extends UnicastRemoteObject implements GhidraServerHan
 	}
 
 	@Override
-	public void checkCompatibility(int serverInterfaceVersion) throws RemoteException {
-		if (serverInterfaceVersion > INTERFACE_VERSION) {
+	public void checkCompatibility(int minServerInterfaceVersion) throws RemoteException {
+		if (minServerInterfaceVersion > INTERFACE_VERSION) {
 			throw new RemoteException(
 				"Incompatible server interface, a newer Ghidra Server version is required.");
 		}
-		else if (serverInterfaceVersion < INTERFACE_VERSION) {
+		else if (minServerInterfaceVersion < MINIMUM_INTERFACE_VERSION) {
 			throw new RemoteException(
 				"Incompatible server interface, the minimum supported Ghidra version is " +
 					MIN_GHIDRA_VERSION);
@@ -402,7 +412,7 @@ public class GhidraServer extends UnicastRemoteObject implements GhidraServerHan
 	/**
 	 * Display an optional message followed by usage syntax.
 	 *
-	 * @param msg
+	 * @param msg optional message (may be null)
 	 */
 	private static void displayUsage(String msg) {
 		if (msg != null) {
@@ -413,7 +423,7 @@ public class GhidraServer extends UnicastRemoteObject implements GhidraServerHan
 
 	private static void displayHelp() {
 
-		try (InputStream in = ResourceManager.getResourceAsStream(HELP_FILE)) {
+		try (InputStream in = GhidraServer.class.getResourceAsStream(HELP_FILE)) {
 			List<String> lines = FileUtilities.getLines(in);
 			lines.stream().forEach(s -> System.out.println(s));
 		}
@@ -492,7 +502,7 @@ public class GhidraServer extends UnicastRemoteObject implements GhidraServerHan
 		ResourceFile serverRoot = new ResourceFile(Application.getInstallationDirectory(),
 			SystemUtilities.isInDevelopmentMode() ? "ghidra/Ghidra/RuntimeScripts/Common/server"
 					: "server");
-		if (serverRoot == null || serverRoot.getFile(false) == null) {
+		if (serverRoot.getFile(false) == null) {
 			System.err.println(
 				"Failed to resolve installation root directory!: " + serverRoot.getAbsolutePath());
 			System.exit(-1);
@@ -533,7 +543,7 @@ public class GhidraServer extends UnicastRemoteObject implements GhidraServerHan
 
 		// Network name resolution disabled by default
 		InetNameLookup.setLookupEnabled(false);
-		
+
 		// Initialize application
 		try {
 			ApplicationLayout layout = new GhidraServerApplicationLayout();
@@ -717,10 +727,11 @@ public class GhidraServer extends UnicastRemoteObject implements GhidraServerHan
 			System.exit(-1);
 		}
 
-		Application.initializeLogging(new File(serverRoot, "server.log"), null);
+		File serverLogFile = new File(serverRoot, "server.log");
+		Application.initializeLogging(serverLogFile, serverLogFile);
 
 		// In the absence of module initialization - we must invoke directly
-		SSLContextInitializer.initialize();
+		DefaultSSLContextInitializer.initialize();
 
 		log = LogManager.getLogger(GhidraServer.class); // init log *after* initializing log system
 
@@ -741,12 +752,12 @@ public class GhidraServer extends UnicastRemoteObject implements GhidraServerHan
 			// Ensure that remote access hostname is properly set for RMI registration
 			String hostname = initRemoteAccessHostname();
 
-			if (ApplicationKeyManagerFactory.getPreferredKeyStore() == null) {
+			if (DefaultKeyManagerFactory.getPreferredKeyStore() == null) {
 				// keystore has not been identified - use self-signed certificate
-				ApplicationKeyManagerFactory.setDefaultIdentity(
-					new X500Principal("CN=GhidraServer"));
+				DefaultKeyManagerFactory.setDefaultIdentity(new X500Principal("CN=GhidraServer"));
+				DefaultKeyManagerFactory.addSubjectAlternativeName(hostname);
 			}
-			if (!ApplicationKeyManagerFactory.initialize()) {
+			if (!DefaultKeyManagerFactory.initialize()) {
 				log.fatal("Failed to initialize PKI/SSL keystore");
 				System.exit(0);
 				return;
@@ -789,17 +800,28 @@ public class GhidraServer extends UnicastRemoteObject implements GhidraServerHan
 			log.info(
 				"   Anonymous server access: " + (allowAnonymousAccess ? "enabled" : "disabled"));
 
-			log.info(SystemUtilities.getUserName() + " starting Ghidra Server...");
+			String enabledCiphers = System.getProperty(TLS_ENABLED_CIPHERS_PROPERTY);
+			if (enabledCiphers != null) {
+				String[] cipherList = enabledCiphers.split(",");
+				log.info("   Enabled cipher suites:");
+				for (String s : cipherList) {
+					log.info("       " + s);
+				}
+			}
 
-			serverSocketFactory = new SslRMIServerSocketFactory(null, null, authMode == PKI_LOGIN) {
+			serverSocketFactory = new SslRMIServerSocketFactory(null, getEnabledTlsProtocols(),
+				authMode == PKI_LOGIN) {
 				@Override
 				public ServerSocket createServerSocket(int port) throws IOException {
 					return new GhidraSSLServerSocket(port, bindAddress, getEnabledCipherSuites(),
 						getEnabledProtocols(), getNeedClientAuth());
 				}
+
 			};
 			clientSocketFactory = new SslRMIClientSocketFactory();
 
+			log.info(SystemUtilities.getUserName() + " starting Ghidra Server...");
+
 			GhidraServer svr = new GhidraServer(serverRoot, authMode, loginDomain,
 				nameCallbackAllowed, altSSHLoginAllowed, defaultPasswordExpiration,
 				allowAnonymousAccess, autoProvision, jaasConfigFile);
@@ -824,6 +846,21 @@ public class GhidraServer extends UnicastRemoteObject implements GhidraServerHan
 		}
 	}
 
+	private static String[] getEnabledTlsProtocols() {
+		String protocolList = System.getProperty(TLS_SERVER_PROTOCOLS_PROPERTY);
+		if (protocolList != null) {
+
+			log.info("   Enabled protocols: " + protocolList);
+
+			String[] protocols = protocolList.split(";");
+			for (int i = 0; i < protocols.length; i++) {
+				protocols[i] = protocols[i].trim();
+			}
+			return protocols;
+		}
+		return null;
+	}
+
 	static synchronized void stop() {
 		if (server == null) {
 			throw new IllegalStateException("Invalid Stop request, Server is not running");
@@ -839,4 +876,122 @@ public class GhidraServer extends UnicastRemoteObject implements GhidraServerHan
 		return clientSocketFactory;
 	}
 
+	private static void setGlobalSerializationFilter() throws IOException {
+
+		// NOTE: Serialization filter may need to be disabled when profiling with VisualVM
+		String disabledStr = System.getProperty(SERIALIZATION_FILTER_DISABLED_PROPERTY);
+		if (Boolean.valueOf(disabledStr)) {
+			return;
+		}
+
+		ObjectInputFilter patternFilter = readSerialFilterPatternFile();
+
+		ObjectInputFilter filter = new ObjectInputFilter() {
+
+			@Override
+			public Status checkInput(FilterInfo info) {
+
+				Class<?> clazz = info.serialClass();
+
+				// Give serial filter patterns first shot
+				Status status = patternFilter.checkInput(info);
+				if (status != Status.UNDECIDED) {
+					if (status == Status.REJECTED) {
+						return serialReject(info, "failed by serial.filter pattern");
+					}
+					return status;
+				}
+
+				if (clazz == null) {
+					return Status.ALLOWED;
+				}
+
+				Class<?> componentType = clazz.getComponentType();
+				if (componentType != null && componentType.isPrimitive()) {
+					return Status.ALLOWED; // allow all primitive arrays
+				}
+
+				return serialReject(info, "not allowed");
+			}
+
+			private Status serialReject(FilterInfo info, String reason) {
+				String clientHost = RepositoryManager.getRMIClient();
+				StringBuilder buf = new StringBuilder();
+				buf.append("Rejected class serialization");
+				if (clientHost != null) {
+					buf.append(" from ");
+					buf.append(clientHost);
+				}
+				buf.append("(");
+				buf.append(reason);
+				buf.append(")");
+
+				Class<?> serialClass = info.serialClass();
+				if (serialClass != null) {
+					buf.append(": ");
+					buf.append(serialClass.getCanonicalName());
+					buf.append(" ");
+					if (serialClass.getComponentType() != null) {
+						buf.append("(");
+						buf.append("array-length=");
+						buf.append(info.arrayLength());
+						buf.append(")");
+					}
+				}
+
+				log.error(buf.toString());
+				return Status.REJECTED;
+			}
+
+		};
+
+		// Install global serial class filter
+		ObjectInputFilter.Config.setSerialFilter(filter);
+	}
+
+	/**
+	 * Read serial.filter file content removing any comments and newlines and generate 
+	 * corresponding {@link ObjectInputFilter}.  See {@link java.io.ObjectInputFilter.Config#createFilter(String)}
+	 * for filter syntax.
+	 * @return serial filter content 
+	 * @throws IOException if file error occurs
+	 */
+	private static ObjectInputFilter readSerialFilterPatternFile() throws IOException {
+
+		File serialFilterFile = Application.getModuleDataFile(SERIAL_FILTER_FILE).getFile(false);
+		if (serialFilterFile == null) {
+			// jar mode not supported
+			throw new FileNotFoundException(SERIAL_FILTER_FILE + " not found");
+		}
+		try {
+			StringBuilder buf = new StringBuilder();
+			try (FileReader fr = new FileReader(serialFilterFile);
+					BufferedReader r = new BufferedReader(fr)) {
+
+				for (String line = r.readLine(); line != null; line = r.readLine()) {
+					int ix = line.indexOf('#');
+					if (ix >= 0) {
+						// strip comment
+						line = line.substring(0, ix);
+					}
+					line = line.trim();
+					if (line.length() == 0) {
+						continue;
+					}
+					if (!line.endsWith(";")) {
+						throw new IllegalArgumentException(
+							"all filter statements must end with `;`");
+					}
+					if (line.length() != 0) {
+						buf.append(line);
+					}
+				}
+			}
+			return ObjectInputFilter.Config.createFilter(buf.toString());
+		}
+		catch (Exception e) {
+			throw new IOException("Failed to parse " + SERIAL_FILTER_FILE, e);
+		}
+	}
+
 }
