@@ -4,9 +4,9 @@
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -21,22 +21,23 @@ import javax.swing.event.TableModelEvent;
 import javax.swing.table.TableModel;
 
 import docking.widgets.table.sort.DefaultColumnComparator;
-import docking.widgets.table.sort.RowToColumnComparator;
+import docking.widgets.table.sort.RowBasedColumnComparator;
+import ghidra.util.Msg;
 import ghidra.util.Swing;
 import ghidra.util.datastruct.WeakDataStructureFactory;
 import ghidra.util.datastruct.WeakSet;
 
 /**
  * Table models should extends this model when they want sorting, potentially across multiple
- * columns, but do not want Threading or do not work on Program-related data (Address, 
+ * columns, but do not want Threading or do not work on Program-related data (Address,
  * ProgramLocations, etc...).
  * <p>
- * In order to define custom comparators for a column, simply override 
+ * In order to define custom comparators for a column, simply override
  * {@link #createSortComparator(int)}.  Otherwise, a default comparator will be created for you.
- * 
- * <p>Note on sorting: it is possible that the user can disable sorting by de-selecting all 
- * sorted columns.   This can also be achieved programmatically by calling 
- * {@link #setTableSortState(TableSortState)} with a value of 
+ *
+ * <p>Note on sorting: it is possible that the user can disable sorting by de-selecting all
+ * sorted columns.   This can also be achieved programmatically by calling
+ * {@link #setTableSortState(TableSortState)} with a value of
  * {@link TableSortState#createUnsortedSortState()}.
  *
  * @param <T> The row type upon which the table is based
@@ -92,11 +93,11 @@ public abstract class AbstractSortedTableModel<T> extends AbstractGTableModel<T>
 	}
 
 	/**
-	 * Returns the index of the given row object in this model; -1 if the model does not contain
-	 * the given object.  
-	 * 
-	 * <p>Warning: if the this model has no sort applied, then performance will be O(n).  If 
-	 * sorted, then performance is O(log n).  You can call {@link #isSorted()} to know when 
+	 * Returns the index of the given row object in this model; a negative value if the model
+	 * does not contain the given object.
+	 *
+	 * <p>Warning: if the this model has no sort applied, then performance will be O(n).  If
+	 * sorted, then performance is O(log n).  You can call {@link #isSorted()} to know when
 	 * this will happen.
 	 */
 	@Override
@@ -132,6 +133,9 @@ public abstract class AbstractSortedTableModel<T> extends AbstractGTableModel<T>
 
 	@Override
 	public int getPrimarySortColumnIndex() {
+		if (sortState.isUnsorted()) {
+			return -1;
+		}
 		return sortState.iterator().next().getColumnModelIndex();
 	}
 
@@ -154,20 +158,50 @@ public abstract class AbstractSortedTableModel<T> extends AbstractGTableModel<T>
 			return false; // more columns than we have
 		}
 
-		for (int i = 0; i < columnCount; i++) {
-			ColumnSortState state = tableSortState.getColumnSortState(i);
-			if (state == null) {
-				continue; // no sort state for this column--nothing to validate
+		// verify the requested columns are sortable
+		for (ColumnSortState state : tableSortState) {
+
+			int index = state.getColumnModelIndex();
+			if (!isSortable(index)) {
+				return false; // the state wants to sort on an unsortable column
 			}
 
-			if (!isSortable(i)) {
-				return false; // the state wants to sort on an unsortable column
+			if (index >= columnCount) {
+				return false; // requested a column that is larger than the number of columns
 			}
 		}
 
 		return true;
 	}
 
+	/**
+	 * A convenience method that will take a given sort state and remove from it any columns
+	 * that cannot be sorted.  This is useful if the client is restoring a sort state that
+	 * contains columns that have been removed or are no longer sortable (such as during major
+	 * table model rewrites).
+	 *
+	 * @param state the state
+	 * @return the updated state
+	 */
+	protected TableSortState cleanupTableSortState(TableSortState state) {
+
+		int columnCount = getColumnCount();
+		TableSortStateEditor editor = new TableSortStateEditor(state);
+		int n = editor.getSortedColumnCount();
+		for (int i = 0; i < n; i++) {
+			ColumnSortState ss = editor.getColumnSortState(i);
+			int columnIndex = ss.getColumnModelIndex();
+			if (columnIndex >= columnCount) {
+				editor.removeSortedColumn(columnIndex);
+			}
+			if (!isSortable(columnIndex)) {
+				editor.removeSortedColumn(columnIndex);
+			}
+		}
+
+		return editor.createTableSortState();
+	}
+
 	private void doSetTableSortState(final TableSortState newSortState) {
 		if (newSortState.equals(pendingSortState)) {
 			return; // there is an upcoming sort that matches the current request
@@ -187,20 +221,43 @@ public abstract class AbstractSortedTableModel<T> extends AbstractGTableModel<T>
 		return pendingSortState;
 	}
 
+	/**
+	 * Returns true if there is a pending change to the current sort state
+	 * (this includes a sort state that signals no sort will be applied)
+	 *
+	 * @return true if there is a pending change to the current sort state
+	 */
 	public boolean isSortPending() {
 		return isSortPending;
 	}
 
+	/**
+	 * Returns true if this model has been sorted and does not have a new pending sort that will
+	 * be applied
+	 *
+	 * @return true if sorted
+	 * @see #isSortPending()
+	 */
 	public boolean isSorted() {
 		return !isSortPending && !sortState.isUnsorted();
 	}
 
 	protected TableSortingContext<T> createSortingContext(TableSortState newSortState) {
+
+		if (!isValidSortState(newSortState)) {
+			newSortState = TableSortState.createUnsortedSortState();
+			if (!isDisposed) {
+				Msg.error(this, """
+						"Table '%s' sort is invalid.  Assuming columns have been removed. \
+						Setting unsorted.""".formatted(getName()));
+			}
+		}
+
 		return new TableSortingContext<>(newSortState, getComparatorChain(newSortState));
 	}
 
 	/**
-	 * The default implementation of {@link TableModel#getValueAt(int, int)} that calls the 
+	 * The default implementation of {@link TableModel#getValueAt(int, int)} that calls the
 	 * abstract {@link #getColumnValueForRow(Object, int)}.
 	 */
 	@Override
@@ -210,9 +267,9 @@ public abstract class AbstractSortedTableModel<T> extends AbstractGTableModel<T>
 	}
 
 	/**
-	 * This method is an attempt to help models that forget to call fireTableDataChanged().  It 
-	 * is expected that tables will fire the notification when they are ready to display data, 
-	 * even if they have that data at construction time.  We put this call here so that the 
+	 * This method is an attempt to help models that forget to call fireTableDataChanged().  It
+	 * is expected that tables will fire the notification when they are ready to display data,
+	 * even if they have that data at construction time.  We put this call here so that the
 	 * forgetful subclasses will have their data sorted for them the first time that this table
 	 * tries to render itself.
 	 */
@@ -230,11 +287,11 @@ public abstract class AbstractSortedTableModel<T> extends AbstractGTableModel<T>
 	/**
 	 * A convenience method for subclasses to quickly/efficiently search for the index of a given
 	 * row object <b>that is visible in the GUI</b>.  The <i>visible</i> limitation is due to the
-	 * fact that the data searched is retrieved from {@link #getModelData()}, which may be 
-	 * filtered.  
-	 * 
+	 * fact that the data searched is retrieved from {@link #getModelData()}, which may be
+	 * filtered.
+	 *
 	 * @param rowObject The object for which to search.
-	 * @return the index of the item in the data returned by 
+	 * @return the index of the item in the data returned by
 	 */
 	@Override
 	protected int getIndexForRowObject(T rowObject) {
@@ -242,8 +299,8 @@ public abstract class AbstractSortedTableModel<T> extends AbstractGTableModel<T>
 	}
 
 	/**
-	 * Returns the index for the given object in the given list 
-	 * 
+	 * Returns the index for the given object in the given list
+	 *
 	 * @param rowObject the item
 	 * @param data the data
 	 * @return the index
@@ -267,11 +324,11 @@ public abstract class AbstractSortedTableModel<T> extends AbstractGTableModel<T>
 	/**
 	 * A default sort method that uses the {@link Collections#sort(List, Comparator)} method for
 	 * sorting.  Implementors with reasonably sized data sets can rely on this method.  For data
-	 * sets that can become large, the <tt>ThreadedTableModel</tt> is the recommended base class, 
+	 * sets that can become large, the <code>ThreadedTableModel</code> is the recommended base class,
 	 * as it handles loading/sorting/filtering in a threaded way.
-	 * 
+	 *
 	 * @param data The data to be sorted
-	 * @param sortingContext The context required to sort (it contains the sorting columns, a 
+	 * @param sortingContext The context required to sort (it contains the sorting columns, a
 	 *        comparator for sorting, etc...).
 	 */
 	protected void sort(List<T> data, TableSortingContext<T> sortingContext) {
@@ -299,8 +356,8 @@ public abstract class AbstractSortedTableModel<T> extends AbstractGTableModel<T>
 	}
 
 	/**
-	 * Fires an event to let the listeners (like JTable) know that things have been changed. 
-	 * This method exists so that subclasses have a way to call the various <tt>tableChanged()</tt>
+	 * Fires an event to let the listeners (like JTable) know that things have been changed.
+	 * This method exists so that subclasses have a way to call the various <code>tableChanged()</code>
 	 * methods without triggering this class's overridden version.
 	 * @param dataChanged True signals that the actual data has changed; false signals that the
 	 *        data is the same, with exception that attributes of that data may be different.
@@ -321,14 +378,14 @@ public abstract class AbstractSortedTableModel<T> extends AbstractGTableModel<T>
 
 	/**
 	 * An extension point for subclasses to insert their own comparator objects for their data.
-	 * Subclasses can create comparators for a single or multiple columns, as desired.  
-	 * 
+	 * Subclasses can create comparators for a single or multiple columns, as desired.
+	 *
 	 * @param columnIndex the column index
-	 * @return the comparator 
+	 * @return the comparator
 	 */
 	protected Comparator<T> createSortComparator(int columnIndex) {
-		return new RowToColumnComparator<>(this, columnIndex, new DefaultColumnComparator(),
-			new StringBasedBackupRowToColumnComparator(columnIndex));
+		return new RowBasedColumnComparator<>(this, columnIndex, new DefaultColumnComparator(),
+			new StringBasedBackupRowToColumnComparator());
 	}
 
 	private Comparator<T> createLastResortComparator(ComparatorLink parentChain) {
@@ -340,7 +397,7 @@ public abstract class AbstractSortedTableModel<T> extends AbstractGTableModel<T>
 		// one column sorted and that column is 'reversed', then to provide consistent sorting,
 		// and later searching, we need to reverse the tie-breaker comparator.  Without this,
 		// the tie-breaker always produces the same results, regardless of which direction the
-		// search is going.  That will break any clients that call Collections.reverse() in 
+		// search is going.  That will break any clients that call Collections.reverse() in
 		// order to quickly invert an existing sort.
 		//
 		if (parentChain.primaryComparator instanceof AbstractSortedTableModel.ReverseComparator) {
@@ -374,8 +431,8 @@ public abstract class AbstractSortedTableModel<T> extends AbstractGTableModel<T>
 	}
 
 	/**
-	 * Builds a comparator for the given column sort state while allowing for subclasses to 
-	 * provider their own comparators.  This method also handles directionality of sorting, so 
+	 * Builds a comparator for the given column sort state while allowing for subclasses to
+	 * provider their own comparators.  This method also handles directionality of sorting, so
 	 * that the comparators used can be simple and generic.
 	 */
 	private Comparator<T> getComparator(ColumnSortState columnSortState) {
@@ -442,8 +499,8 @@ public abstract class AbstractSortedTableModel<T> extends AbstractGTableModel<T>
 		public int compare(T t1, T t2) {
 
 			// at this point we compare the rows, since all of the sorting column values are equal
-			// (Warning: due to comparable being specific to the class upon which it is defined, 
-			//           we have to make sure the class is the same to prevent class cast 
+			// (Warning: due to comparable being specific to the class upon which it is defined,
+			//           we have to make sure the class is the same to prevent class cast
 			//           exceptions when the table has mixed implementations of 'T')
 			if (t1 instanceof Comparable && t1.getClass().equals(t2.getClass())) {
 				return ((Comparable) t1).compareTo(t2);
@@ -467,22 +524,16 @@ public abstract class AbstractSortedTableModel<T> extends AbstractGTableModel<T>
 		}
 	}
 
-	private class StringBasedBackupRowToColumnComparator implements Comparator<T> {
-
-		private int sortColumn;
-
-		StringBasedBackupRowToColumnComparator(int sortColumn) {
-			this.sortColumn = sortColumn;
-		}
+	private class StringBasedBackupRowToColumnComparator implements Comparator<Object> {
 
 		@Override
-		public int compare(T t1, T t2) {
-			if (t1 == t2) {
+		public int compare(Object c1, Object c2) {
+			if (c1 == c2) {
 				return 0;
 			}
 
-			String s1 = getColumStringValue(t1);
-			String s2 = getColumStringValue(t2);
+			String s1 = getColumStringValue(c1);
+			String s2 = getColumStringValue(c2);
 
 			if (s1 == null || s2 == null) {
 				return TableComparators.compareWithNullValues(s1, s2);
@@ -491,11 +542,10 @@ public abstract class AbstractSortedTableModel<T> extends AbstractGTableModel<T>
 			return s1.compareToIgnoreCase(s2);
 		}
 
-		private String getColumStringValue(T t) {
+		private String getColumStringValue(Object columnValue) {
 			// just use the toString(), which may or may not produce a good value (this will
 			// catch the cases where the column value is itself a string)
-			Object o = getColumnValueForRow(t, sortColumn);
-			return o == null ? null : o.toString();
+			return columnValue == null ? null : columnValue.toString();
 		}
 	}
 }
